<?php

namespace SupplyTek\Utilities;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Doctrine\ORM\Configuration;
use Zeedhi\Framework\Session\Session;
use SupplyTek\Utilities\SessionHandler;


class ConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {

    /**
     * Static method to update connection params to conect with SAAS base
     * 
     * @param array $connectionParams Array with connnection params
     * @param Configuration $entityMetaDataConfig Object with Entity Config
     * @param EventManager $eventManager Object to manipulate connections
     * @param SessionHandler $session Object to manipulate connections
     * @param Session $zdSession Object to manipulate connections
     * 
     * 
     * @return Connection
     * 
     * @access public
     * @author Pedro Henrique Vilaça Valverde
     */
    public static function updateConnectionParams(array $connectionParams, $config = null, $eventManager = null, SessionHandler $session = null, Session $zdSession = null){
        
        $nrOrg = false;
        if($session) {
            $nrOrg = $session->getSessionData('nrorg');
            
            self::setCurrentDBName($connectionParams, $zdSession);
        }
        
        if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
            $connectionParams = self::factoryVPDConnectionParams($connectionParams, $nrOrg);
        }
        
        if(empty($connectionParams["password"])) {
            $connectionParams["password"] = '';
        } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
            $cryptKey = isset($connectionParams["cryptKey"]) ? $connectionParams["cryptKey"] : null;
            $connectionParams["password"] = Utilities::zeedhiDecrypt($connectionParams["password"], $cryptKey);
            $connectionParams["isEncrypted"] = false;
        }
        
        $connection = DriverManager::getConnection($connectionParams, $config, $eventManager);
        self::insertSQLWebTunning($connection, $zdSession);
        return $connection;
    }

    /**
     * Set currennt DBNAM in session for use in login session
     * 
     * @param array $connectionParams Array with connnection params
     * @param Session $zdSession Object to manipulate connections
     * 
     * @return void
     * 
     * @access protected
     * @author Pedro Henrique Vilaça Valverde
     */
    protected static function setCurrentDBName($connectionParams, $zdSession) {
        if(isset($connectionParams['sessionMode'])) {
            $encryptedDBName = Utilities::zeedhiEncrypt($connectionParams['dbname']);
            $zdSession->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName));
        } else {
            $encryptedUser = Utilities::zeedhiEncrypt($connectionParams['user']);
            $zdSession->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '',$encryptedUser));
        }
    }

    /**
     * Change connectionParams to adapt to each situation such as SAAS connections
     * 
     * @param array $connectionParams Array with connnection params
     * @param string $nrOrg String with nrorg number
     * 
     * @return array
     * 
     * @access protected
     * @author Pedro Henrique Vilaça Valverde
     */
    protected static function factoryVPDConnectionParams($connectionParams, $nrOrg) {
        if (isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
            $connectionParams['user']        = '/';
            $connectionParams['password']    = '';
            $connectionParams['dbname']      = 'USR_ORG_' . $nrOrg;
            $connectionParams['sessionMode'] = -2147483648;
            
            unset($connectionParams['host']);
            unset($connectionParams['port']);
        } else {
            $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
            $connectionParams['password'] = $connectionParams['vpdPassword'];
            if (isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                $connectionParams['dbname']  = $connectionParams['vpdDBNAME'];
                $connectionParams['host']    = $connectionParams['vpdHost'];
                $connectionParams['port']    = $connectionParams['vpdPort'];
                $connectionParams['service'] = $connectionParams['vpdService'];
                unset($connectionParams['sessionMode']);
            }
        }
        
        return $connectionParams;
    }

    /**
     * Config SQL Web Tunneing pararmeters
     * 
     * @param Connection $connection
     * @param Session $zdSession
     * 
     * @return void
     * 
     * @access protected
     * @author Pedro Henrique Vilaça Valverde
     */
    public static function insertSQLWebTunning($connection, $zdSession) {
        try {
            if(Utilities::getUseSQLWebTunningParameter()) {
                $query = "INSERT INTO SQLWEBTUNNING (DSREMOTEADDR, DSREMOTEHOST, DSREMOTEAPPL, DSSERVERRDM,
                                                     DSSERVERDLL,  DTSTARTDATE,  DSREMOTEOPER, DSSERVMETHOD)
                               VALUES (:DSREMOTEADDR, :DSREMOTEHOST, :DSREMOTEAPPL, :DSSERVERRDM,
                                       :DSSERVERDLL,  SYSDATE,       :DSREMOTEOPER, :DSSERVMETHOD)";

				if(isset($_SERVER['HTTP_HOST'])) {
					$host = $_SERVER['HTTP_HOST'];
				} else if(isset($_SERVER['SERVER_NAME'])) {
					$host = $_SERVER['SERVER_NAME'];
				} else {
					$host = 'EMPTY_HOST';
				}
				if(isset($_SERVER['REQUEST_URI'])) {
					$requestUri = explode('index.php', trim($_SERVER['REQUEST_URI']));
					$route      = array_pop($requestUri);	
				} else {
					$route = 'EMPTY_ROUTE';
				}
				
                $cdoperador = $zdSession && $zdSession->has('CDOPERADOR') ? $zdSession->get('CDOPERADOR') : 'SESSAO_N_INI';
                $hostArr    = explode('.', $host);
                $hostInit   = array_shift($hostArr);
                
                $params = array (
                    'DSREMOTEADDR' => trim(substr(gethostbyname($host), 0, 15)),
                    'DSREMOTEHOST' => trim(substr($hostInit, 0, 15)),
                    'DSREMOTEAPPL' => strval(Utilities::getProjectIdParameter()),
                    'DSSERVERRDM'  => trim(implode('.', $hostArr)),
                    'DSSERVERDLL'  => 'ZEEDHI_REQUEST',
                    'DSREMOTEOPER' => $cdoperador,
                    'DSSERVMETHOD' => substr($route, 0, 30)
                );
                $connection->executeQuery($query, $params);
            }
        } catch(\Exception $e) {}
    }
}